# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_harvest_data.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

import logging
from logging import getLogger
import requests
import time

from soracom_debug_util import SoracomDummyResponse

logger = getLogger(__name__)

class SoracomHarvestData:
    def __init__(self):
        self.scheme = 'http'
        self.host = 'harvest.soracom.io'
        self.port = 0

    def uploadData(self, encode_json_data, timestamp=time.time(), debug=False):
        "post encoded json data"

        try:
            url = self.scheme
            url += "://"
            url += self.host
            if self.port > 0:
                url += ":"
                url += str(self.port)
            url += "/"
            logger.info('url: %s', url)
            logger.info('Harvest data upload data: %s timestamp: %s', encode_json_data, int(round(timestamp * 1000)))
            if debug:
                logger.debug("debug is set and data: %s", encode_json_data)
                return SoracomDummyResponse()
            response = requests.post(
                url, data=encode_json_data,
                headers={'Content-Type': 'application/json', 'x-soracom-timestamp': str(int(round(timestamp * 1000)))})
            logger.debug('response from Harvest Data: %s', response)
            return response
        except Exception as err:
            logger.error("can't upload data by %s", err)
            return None
