# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_harvest_files.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

import logging
from logging import getLogger
import os
import re
import requests
import json

from soracom_debug_util import SoracomDummyResponse

logger = getLogger(__name__)

DEBUG_LOCAL_SAVEPATH = "/tmp/io.soracom.harvestfiles"


class SoracomHarvestFiles:
    def __init__(self):
        self.scheme = "http"
        self.host = "harvest-files.soracom.io"
        self.path = "/"
        self.port = 0

    def uploadData(self, name, imageData, contentType='image/jpeg', debug=False):
        try:
            url = self.scheme
            url += "://"
            url += self.host
            if self.port > 0:
                url += ":"
                url += str(self.port)
            if len(name) > 0 and name[0] != self.path:
                url += self.path
            url += name
            logger.info('url=%s', url)
            response = False
            headers = {'Content-Type': contentType}

            if debug:
                logger.info('save file=%s', DEBUG_LOCAL_SAVEPATH + name)
                dirname = os.path.dirname(DEBUG_LOCAL_SAVEPATH + name)
                logger.info(dirname)
                os.makedirs(dirname, exist_ok=True)
                with open(DEBUG_LOCAL_SAVEPATH + name, mode="wb") as f:
                    f.write(imageData)
                return SoracomDummyResponse()
            else:
                response = requests.put(url, data=imageData, headers=headers)
            return response
        except Exception as err:
            logger.error("can't upload data by %s", err)
            return None

    def deleteData(self, name, debug=False):
        response = True
        try:
            url = self.scheme
            url += "://"
            url += self.host
            if self.port > 0:
                url += ":"
                url += str(self.port)
            url += self.path
            url += name
            logger.info('url: %s', url)
            response = False
            headers = {}

            if debug:
                logger.info('remove file: %s', DEBUG_LOCAL_SAVEPATH + name)
                os.remove(DEBUG_LOCAL_SAVEPATH + name)
                return SoracomDummyResponse()
            else:
                response = requests.delete(url, headers=headers)
            return response
        except Exception:
            return None

    def downloadFile(self, name):
        "download file"
        try:
            if name and re.match('^http', name):
                url = name
            else:
                url = self.scheme
                url += "://"
                url += self.host
            if self.port > 0:
                url += ":"
                url += str(self.port)
                url += self.path
                url += name
            logger.info('url: %s', url)
            response = False
            headers = {}
            if self.debug:
                logger.info('read file: %s', DEBUG_LOCAL_SAVEPATH + name)
                with open(DEBUG_LOCAL_SAVEPATH + name, mode="rb") as f:
                    data = f.read()
                    logger.info('data file=%s', data)
                return SoracomDummyResponse(data)
            else:
                response = requests.get(url, headers=headers)
            return response
        except Exception:
            return None
