# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_image_analize_thread_controller.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

from time import sleep
from logging import getLogger
from soracom_image_analize_thread import SoracomImageAnalizeThread

logger = getLogger(__name__)

class SoracomImageAnalizeThreadController:
    def __init__(self, jobs, useMultiprocessing=True,
                 analizeImagePoolModule=None, fifo=True, multiprocessingTimeout=10):
        self.image_analizes_jobs = jobs
        self.image_analizes_works_index = 0
        self.image_analizes_working = 0
        self.image_analizes = []
        for i in range(self.image_analizes_jobs):
            self.image_analizes.append(SoracomImageAnalizeThread(
                useMultiprocessing, analizeImagePoolModule, multiprocessingTimeout))
        self.fifo = fifo

    def getResult(self):
        is_working = True
        result = None
        index = (self.image_analizes_works_index - self.image_analizes_working) % \
                self.image_analizes_jobs
        if self.image_analizes_working >= self.image_analizes_jobs:
            logger.debug('image_analizes[%d] image_analizes_works_index: %d \
                image_analizes_working: %d', \
                    index, self.image_analizes_works_index, self.image_analizes_working)
            while is_working is True:
                is_working = self.image_analizes[index].getWorking()
                if is_working and not self.fifo:
                    index = (index + 1) % self.image_analizes_jobs
                sleep(0.01)
            result = self.image_analizes[index].getResult()
            self.image_analizes[index].resetAnalize()
            self.image_analizes_working = \
                self.image_analizes_working - 1 if self.image_analizes_working > 0 else 0
        logger.debug('get result of index: %d, image_analizes_works_index: %d \
            image_analizes_working: %d', \
                index, self.image_analizes_works_index, self.image_analizes_working)
        return result

    def setImage(self, analize_type, image, image_id=-1, resources=None, options=None):
        logger.debug('setImage start image_id: %d image_analizes_works_index: %d \
            image_analizes_working: %d',\
                image_id, self.image_analizes_works_index, self.image_analizes_working)
        is_working = True
        index = (self.image_analizes_works_index -
                 self.image_analizes_working) % self.image_analizes_jobs
        if self.image_analizes_working >= self.image_analizes_jobs or not self.fifo:
            while is_working is True:
                is_working = self.image_analizes[index].getWorking()
                if is_working and not self.fifo:
                    index = (index + 1) % self.image_analizes_jobs
                sleep(0.01)
        if self.fifo:
            index = self.image_analizes_works_index % \
               self.image_analizes_jobs
        logger.debug('image_analizes[%d]', index)

        self.image_analizes[index].startAnalize(
            analize_type, image, image_id, resources, options)
        self.image_analizes_works_index += 1
        self.image_analizes_works_index %= self.image_analizes_jobs * 2
        self.image_analizes_working += 1
        logger.debug('index: %d, fifo: %s image_analizes_jobs: %d image_analizes_works_index: %d, image_analizes_working: %d image_id: %d',
                     index, self.fifo, self.image_analizes_jobs, self.image_analizes_works_index, self.image_analizes_working, image_id)
        logger.debug('setImage end')


    def putImage(self, analize_type, image, image_id=-1, resources=None, options=None):
        result = self.getResult()
        self.setImage(analize_type, image, image_id, resources, options)
        return result
