# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_movie_upload.py
# Copyright       : 2015-2021 SORACOM, INC.
# ======================================================

import os
from datetime import datetime
from time import sleep
import traceback
from logging import getLogger
import requests
from soracom_harvest_files import SoracomHarvestFiles
from soracom_harvest_data import SoracomHarvestData
from soracom_data_upload_path_util import SoracomDataUploadPathUtil

logger = getLogger(__name__)

# Harvest Files and Harvest Movie
HARVEST_FILES = SoracomHarvestFiles()
HARVEST_DATA = SoracomHarvestData()

# SORACOM_MOVIE = SoracomMovie()
class SoracomMovieUpload():
    def __init__(self):
        pass

    def uploadMovie(self, device_id, timestamp, data_type, data=None, file_path=None,
                   file_id=-1, resources=None, options=None):
        logger.info("########### SoracomMovieUpload.uploadMovie() \
            device_id: %s data_type: %s resources: %s options: %s file_path: %s timestamp: %s", \
            device_id, data_type, resources, options, file_path, timestamp)
        movie_file = None
        logger.debug("uploadMovie options=%s", options)
        if data_type == "movie":
            data = self.downloadMovie(file_path)
        result = None

        try:
            harvest_file_upload_paths = []
            option_movie = options.get('movie', None)
            debug = options.get('debug', False)
            if data is not None:
                index = 0
                option_movie_top_dir_name = option_movie.get('topDirName', '')
                option_movie_dir_type = option_movie.get('dirType', 'default')
                option_movie_filename_type = option_movie.get('filenameType', 'default')
                if data_type == 'index':
                    option_movie_content_type = option_movie.get('contentTypeIndex', None)
                else:
                    option_movie_content_type = option_movie.get('contentType', None)
                soracom_data_upload_path_util_image = SoracomDataUploadPathUtil(
                        option_movie_top_dir_name, device_id, "",
                        "", "", timestamp,
                        dir_type=option_movie_dir_type, filename_type=option_movie_filename_type)
                dir_name = soracom_data_upload_path_util_image.getDirPath(datetime.fromtimestamp(timestamp))
                logger.info('dir_name = %s', dir_name)
                if data_type == "movie":
                    dir_name = os.path.join(dir_name, "ts")
                full_path = os.path.join(dir_name, os.path.basename(file_path))
                response = \
                    HARVEST_FILES.uploadData(full_path, data, option_movie_content_type, debug)
                harvest_file_upload_paths.append(full_path)
                if response is not None:
                    if response.status_code != 200 and response.status_code != 201:
                        result = {}
                        result['error'] = 'images'
                        result['response'] = response
                        result['index'] = index
                        return result
                    else:
                        result = {}
                        result['error'] = 'images'
                        result['index'] = index
                        return result
                index += 1
        except Exception as error:
            logger.error(traceback.format_exc())
            logger.error(error)
            result = {}
            result['error'] = 'exception'
            result['exception'] = error
        return result

    def downloadMovie(self, file_path):
        try:
            response = requests.get(
                file_path
            )
            logger.debug('response from movie download: %d', response.status_code)
        except Exception as error:
            logger.error("can't down load movie")
            logger.error(error)
            return None
        return response.content

SORACOM_DATA_UPLOAD = None

def run(device_id, timestamp, dataType, data, file_path, file_id, resources, options):
    logger.info("SoracomMovieUpload.run()")

    global SORACOM_DATA_UPLOAD
    if SORACOM_DATA_UPLOAD is None:
        SORACOM_DATA_UPLOAD = SoracomMovieUpload()

    result = SORACOM_DATA_UPLOAD.uploadMovie(
        device_id, timestamp, dataType, data, file_path, file_id, resources, options)
    return result
