# ======================================================
# Project Name    : SORACOM Mosaic CameraApp0 Sample
# File Name       : soracom_movie_upload_thread_controller.py
# Copyright       : 2015-2020 SORACOM, INC.
# ======================================================

from time import sleep
from logging import getLogger
from soracom_movie_upload_thread import SoracomMovieUploadThread

logger = getLogger(__name__)

class SoracomMovieUploadThreadController:
    def __init__(self, jobs, useMultiprocessing=True,
                 uploadDataPoolModule=None, fifo=True, multiprocessingTimeout=10):
        self.movie_uploads_jobs = jobs
        self.movie_uploads_works_index = 0
        self.movie_uploads_working = 0
        self.movie_uploads = []
        for i in range(self.movie_uploads_jobs):
            self.movie_uploads.append(SoracomMovieUploadThread(
                useMultiprocessing, uploadDataPoolModule, multiprocessingTimeout))
        self.fifo = fifo

    def getResult(self):
        is_working = True
        result = None
        index = (self.movie_uploads_works_index - self.movie_uploads_working) % \
                self.movie_uploads_jobs
        if self.movie_uploads_working >= self.movie_uploads_jobs:
            logger.debug('movie_uploads[%d] movie_uploads_works_index: %d movie_uploads_working:%d',\
                        index, self.movie_uploads_works_index, self.movie_uploads_working)
            while is_working is True:
                is_working = self.movie_uploads[index].getWorking()
                if is_working and not self.fifo:
                    index = (index + 1) % self.movie_uploads_jobs
                sleep(0.01)
            result = self.movie_uploads[index].getResult()
            self.movie_uploads[index].resetUpload()
            self.movie_uploads_working = \
                self.movie_uploads_working - 1 if self.movie_uploads_working > 0 else 0
        logger.debug('get result of index: %d, movie_uploads_works_index: %d \
            movie_uploads_working: %d result: %s',\
                      index, self.movie_uploads_works_index, self.movie_uploads_working, result)
        return result

    def setData(self, device_id, timestamp, movie_type, data, file_path, file_id=-1, resources=None, options=None):
        logger.debug('setData start device_id: %s timestamp: %s file_id: %d \
            movie_uploads_works_index: %d movie_uploads_working: %d', \
                device_id, str(timestamp), file_id, self.movie_uploads_works_index, \
                    self.movie_uploads_working)
        is_working = True
        index = (self.movie_uploads_works_index -
                 self.movie_uploads_working) % self.movie_uploads_jobs
        if self.movie_uploads_working >= self.movie_uploads_jobs or not self.fifo:
            logger.debug('movie_uploads[%d]', index)
            while is_working is True:
                is_working = self.movie_uploads[index].getWorking()
                if is_working and not self.fifo:
                    index = (index + 1) % self.movie_uploads_jobs
                sleep(0.01)
        if self.fifo:
            index = self.movie_uploads_works_index % self.movie_uploads_jobs
        self.movie_uploads[index].startUpload(
            device_id, timestamp, movie_type, data, file_path, file_id, resources, options)
        self.movie_uploads_works_index += 1
        self.movie_uploads_works_index %= self.movie_uploads_jobs * 2
        self.movie_uploads_working += 1
        logger.debug("movie_uploads_works_index: %d, movie_uploads_working: %d file_id: %d",
                     self.movie_uploads_works_index, self.movie_uploads_working, file_id)
        logger.debug('setData end')


    def putData(self, device_id, timestamp, movie_type, data, file_path,
                file_id=-1, resources=None, options=None):
        result = self.getResult()
        self.setData(device_id, timestamp, movie_type, data, file_path, file_id, resources, options)
        return result
